#pragma semicolon 1
#include <sourcemod>
#include <sdktools>
#include <sdkhooks>
#include <vip_core>
#include <clientprefs>

public Plugin:myinfo =
{
	name = "[VIP] Nade Models (CSS)",
	author = "R1KO",
	version = "1.0.0"
};

new const String:g_sFeature[] = "NadeModels";
new const String:g_sFeatureMenu[] = "NadeModels_Menu";

#define HE		0
#define FB		1
#define SG		2

#define SIZE	3

new Handle:g_hMenu, Handle:g_hCookie[SIZE], Handle:g_hKeyValues;

public OnPluginStart()
{
	g_hCookie[HE] = RegClientCookie("VIP_HE_MODEL", "VIP_HE_MODEL", CookieAccess_Private);
	g_hCookie[FB] = RegClientCookie("VIP_FB_MODEL", "VIP_FB_MODEL", CookieAccess_Private);
	g_hCookie[SG] = RegClientCookie("VIP_SG_MODEL", "VIP_SG_MODEL", CookieAccess_Private);
	
	g_hMenu = CreateMenu(Handler_NadesMenu, MenuAction_Select|MenuAction_Display|MenuAction_Cancel|MenuAction_DisplayItem);
	SetMenuExitBackButton(g_hMenu, true);
	
	AddMenuItem(g_hMenu, "DisableAll", "DisableAll");
	
	AddMenuItem(g_hMenu, "HE", "HE");
	AddMenuItem(g_hMenu, "FB", "FB");
	AddMenuItem(g_hMenu, "SG", "SG");

	LoadTranslations("vip_nade_models.phrases");
	LoadTranslations("vip_modules.phrases");
}

public VIP_OnVIPLoaded()
{
	VIP_RegisterFeature(g_sFeature, BOOL);
	VIP_RegisterFeature(g_sFeatureMenu, _, SELECTABLE, OnSelectItem, _, OnDrawItem);
}

public OnMapStart()
{
	if(g_hKeyValues != INVALID_HANDLE)
	{
		CloseHandle(g_hKeyValues);
	}

	decl String:sBuffer[256];
	g_hKeyValues = CreateKeyValues("NadeModels");

	if (FileToKeyValues(g_hKeyValues, "addons/sourcemod/data/vip/modules/nade_models.ini") == false)
	{
		CloseHandle(g_hKeyValues);
		SetFailState("Couldn't parse file 'addons/sourcemod/data/vip/modules/nade_models.ini'");
	}

	KvRewind(g_hKeyValues);
	if (KvGotoFirstSubKey(g_hKeyValues))
	{
		decl iType;
		do
		{
			KvSetNum(g_hKeyValues, "valid", 0);
			KvGetString(g_hKeyValues, "model", sBuffer, sizeof(sBuffer));
			if(sBuffer[0] && FileExists(sBuffer, true))
			{
				PrecacheModel(sBuffer, true);
				KvSetNum(g_hKeyValues, "valid", 1);
				
				KvGetString(g_hKeyValues, "type", sBuffer, sizeof(sBuffer));
				iType = 0;
				if(StrContains(sBuffer, "he") != -1)
				{
					iType |= 1;
				}
				if(StrContains(sBuffer, "fb") != -1)
				{
					iType |= 2;
				}
				if(StrContains(sBuffer, "sg") != -1)
				{
					iType |= 4;
				}
				
				KvSetNum(g_hKeyValues, "type_bin", iType);
			}
		}
		while (KvGotoNextKey(g_hKeyValues));
	}
}

new const String:g_sKeyModel[][] =
{
	"HE_MODEL",
	"FB_MODEL",
	"SG_MODEL"
};

new const String:g_sKeyName[][] =
{
	"HE_MODEL_NAME",
	"FB_MODEL_NAME",
	"SG_MODEL_NAME"
};

new const g_iTypes[] =
{
	1,
	2,
	4
};

public VIP_OnVIPClientLoaded(iClient)
{
	if(VIP_GetClientFeatureStatus(iClient, g_sFeature) != NO_ACCESS)
	{
		decl String:sBuffer[256], Handle:hTrie, i;
		hTrie = VIP_GetVIPClientTrie(iClient);
		for(i = 0; i < SIZE; ++i)
		{
			LoadModel(iClient, i, sBuffer, sizeof(sBuffer), hTrie);
		}
	}
}

bool:LoadModel(iClient, index, String:sBuffer[], iMaxLen, &Handle:hTrie)
{
	sBuffer[0] = 0;
	GetClientCookie(iClient, g_hCookie[index], sBuffer, iMaxLen);
	if(sBuffer[0])
	{
		KvRewind(g_hKeyValues);
		if(KvJumpToKey(g_hKeyValues, sBuffer))
		{
			if(KvGetNum(g_hKeyValues, "valid", 0))
			{
				new iType = KvGetNum(g_hKeyValues, "type_bin", 0);
				if(iType == 0 || iType & g_iTypes[index])
				{
					SetTrieString(hTrie, g_sKeyName[index], sBuffer);

					KvGetString(g_hKeyValues, "model", sBuffer, iMaxLen);
					SetTrieString(hTrie, g_sKeyModel[index], sBuffer);
					return true;
				}
			}
		}
	}
	
	return false;
}

public bool:OnSelectItem(iClient, const String:sFeatureName[])
{
	DisplayMenu(g_hMenu, iClient, MENU_TIME_FOREVER);
	return false;
}

public Handler_NadesMenu(Handle:hMenu, MenuAction:action, iClient, Item)
{
	switch(action)
	{
		case MenuAction_Cancel:
		{
			if(Item == MenuCancel_ExitBack) VIP_SendClientVIPMenu(iClient);
		}
		case MenuAction_Display:
		{
	 		decl String:sBuffer[255];
			FormatEx(sBuffer, sizeof(sBuffer), "%T\n ", "NadeModels_MenuTitle", iClient);
 
			SetPanelTitle(Handle:Item, sBuffer);
		}
		case MenuAction_Select:
		{
			if(Item == 0)
			{
				decl Handle:hTrie, i;
				hTrie = VIP_GetVIPClientTrie(iClient);
				for(i = 0; i < SIZE; ++i)
				{
					RemoveFromTrie(hTrie, g_sKeyModel[i]);
					RemoveFromTrie(hTrie, g_sKeyName[i]);
					SetClientCookie(iClient, g_hCookie[i], "");
				}

				DisplayMenu(g_hMenu, iClient, MENU_TIME_FOREVER);
				return 0;
			}

			ShowNadeModelsMenu(iClient, Item-1);
		}
		case MenuAction_DisplayItem:
		{
			decl String:sBuffer[128];
			if(Item)
			{
				decl String:sInfo[32], String:sName[64];
				GetMenuItem(hMenu, Item, sInfo, sizeof(sInfo));
				if(GetTrieString(VIP_GetVIPClientTrie(iClient), g_sKeyName[Item-1], sName, sizeof(sName)))
				{
					FormatEx(sBuffer, sizeof(sBuffer), "%T [%s]", sInfo, iClient, sName);
				}
				else
				{
					FormatEx(sBuffer, sizeof(sBuffer), "%T [-]", sInfo, iClient);
				}
			}
			else
			{
				FormatEx(sBuffer, sizeof(sBuffer), "%T", "DisableAll", iClient);
			}

			return RedrawMenuItem(sBuffer);
		}
	}

	return 0;
}

public OnDrawItem(iClient, const String:sFeatureName[], iStyle)
{
	if(VIP_IsClientFeatureUse(iClient, g_sFeature) == false)
	{
		return ITEMDRAW_DISABLED;
	}

	return iStyle;
}

ShowNadeModelsMenu(iClient, index, Item = 0)
{
	new Handle:hMenu = CreateMenu(NadeModelsMenu_Handler);
	SetGlobalTransTarget(iClient);

	SetMenuExitBackButton(hMenu, true);
	
	decl String:sBuffer[128], String:sClientModel[64], String:sIndex[16];
	GetMenuItem(g_hMenu, index+1, sBuffer, sizeof(sBuffer));
	SetMenuTitle(hMenu, "%t:\n ", sBuffer);
	
	FormatEx(sBuffer, sizeof(sBuffer), "%t\n ", "Disable");
	IntToString(index, sIndex, sizeof(sIndex));
	
	if(GetTrieString(VIP_GetVIPClientTrie(iClient), g_sKeyName[index], sClientModel, sizeof(sClientModel)))
	{
		AddMenuItem(hMenu, sIndex, sBuffer);
	}
	else
	{
		AddMenuItem(hMenu, sIndex, sBuffer, ITEMDRAW_DISABLED);
		sClientModel[0] = 0;
	}

	KvRewind(g_hKeyValues);
	if(KvGotoFirstSubKey(g_hKeyValues))
	{
		decl iType;
		
		do
		{
			if(KvGetNum(g_hKeyValues, "valid", 0))
			{
				iType = KvGetNum(g_hKeyValues, "type_bin", 0);
				if(iType == 0 || iType & g_iTypes[index])
				{
					KvGetSectionName(g_hKeyValues, sBuffer, sizeof(sBuffer));
					if(sClientModel[0] && strcmp(sClientModel, sBuffer) == 0)
					{
						StrCat(sBuffer, sizeof(sBuffer), " [X]");
					}
					AddMenuItem(hMenu, sBuffer, sBuffer);
				}
			}
		} while (KvGotoNextKey(g_hKeyValues));
	}
	
	KvRewind(g_hKeyValues);
	
	if(GetMenuItemCount(hMenu) == 1)
	{
		FormatEx(sBuffer, sizeof(sBuffer), "%t", "NO_MODELS_AVAILABLE");
		AddMenuItem(hMenu, "", sBuffer, ITEMDRAW_DISABLED);
	}
	
	DisplayMenuAtItem(hMenu, iClient, Item, MENU_TIME_FOREVER);
}

public NadeModelsMenu_Handler(Handle:hMenu, MenuAction:action, iClient, Item)
{
	switch(action)
	{
		case MenuAction_End:
		{
			CloseHandle(hMenu);
		}
		case MenuAction_Cancel:
		{
			if(Item == MenuCancel_ExitBack)
			{
				DisplayMenu(g_hMenu, iClient, MENU_TIME_FOREVER);
			}
		}
		case MenuAction_Select:
		{
			decl String:sInfo[256], index, Handle:hTrie;
			GetMenuItem(hMenu, 0, sInfo, sizeof(sInfo));
			StringToIntEx(sInfo, index);

			hTrie = VIP_GetVIPClientTrie(iClient);
			if(Item == 0)
			{
				RemoveFromTrie(hTrie, g_sKeyModel[index]);
				RemoveFromTrie(hTrie, g_sKeyName[index]);
				SetClientCookie(iClient, g_hCookie[index], "");

				DisplayMenu(g_hMenu, iClient, MENU_TIME_FOREVER);
				return 0;
			}

			GetMenuItem(hMenu, Item, sInfo, sizeof(sInfo));

			KvRewind(g_hKeyValues);
			if(KvJumpToKey(g_hKeyValues, sInfo))
			{
				SetTrieString(hTrie, g_sKeyName[index], sInfo);
				SetClientCookie(iClient, g_hCookie[index], sInfo);
				KvGetString(g_hKeyValues, "model", sInfo, sizeof(sInfo));
				SetTrieString(hTrie, g_sKeyModel[index], sInfo);
			}

			ShowNadeModelsMenu(iClient, index, GetMenuSelectionPosition());
		}
	}

	return 0;
}

public OnEntityCreated(iEntity, const String:sClassName[])
{
	if(IsValidEntity(iEntity) && StrContains(sClassName, "_projectile", true) != -1)
	{
		CreateTimer(0.0, Timer_OnSpawnProjectile, EntIndexToEntRef(iEntity), TIMER_FLAG_NO_MAPCHANGE);
	}
}

public Action:Timer_OnSpawnProjectile(Handle:hTimer, any:iEntRef)
{
	new iEntity = EntRefToEntIndex(iEntRef);
	if (iEntity != INVALID_ENT_REFERENCE && IsValidEntity(iEntity))
	{
		new iClient = GetEntPropEnt(iEntity, Prop_Data, "m_hOwnerEntity");
		if(0 < iClient <= MaxClients && IsClientInGame(iClient) && VIP_IsClientVIP(iClient) && VIP_IsClientFeatureUse(iClient, g_sFeature))
		{
			decl String:sModel[256], index;
			GetEntityClassname(iEntity, sModel, 256);
		//	LogMessage("GetEntityClassname: '%s'", sModel);
			switch(sModel[0])
			{
				case 'h':	index = 0;
				case 'f':	index = 1;
				case 's':	index = 2;
				default:	return;
			}

			if(GetTrieString(VIP_GetVIPClientTrie(iClient), g_sKeyModel[index], sModel, sizeof(sModel)))
			{
				SetEntityModel(iEntity, sModel);
			//	DispatchKeyValue(entity, "modelscale", customsmokesize);
			}
		}	
	}
}
